@@ -11,11 +11,13 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
-	"sort"
+	"slices"
+	"strings"
 	"testing"
 	"time"
 
 	"github.com/d4l3k/messagediff"
+
 	"github.com/syncthing/syncthing/lib/db"
 	"github.com/syncthing/syncthing/lib/db/backend"
 	"github.com/syncthing/syncthing/lib/events"
@@ -48,21 +50,19 @@ func globalList(t testing.TB, s *db.FileSet) []protocol.FileInfo {
 	var fs []protocol.FileInfo
 	snap := snapshot(t, s)
 	defer snap.Release()
-	snap.WithGlobal(func(fi protocol.FileIntf) bool {
-		f := fi.(protocol.FileInfo)
-		fs = append(fs, f)
+	snap.WithGlobal(func(fi protocol.FileInfo) bool {
+		fs = append(fs, fi)
 		return true
 	})
 	return fs
 }
 
-func globalListPrefixed(t testing.TB, s *db.FileSet, prefix string) []db.FileInfoTruncated {
-	var fs []db.FileInfoTruncated
+func globalListPrefixed(t testing.TB, s *db.FileSet, prefix string) []protocol.FileInfo {
+	var fs []protocol.FileInfo
 	snap := snapshot(t, s)
 	defer snap.Release()
-	snap.WithPrefixedGlobalTruncated(prefix, func(fi protocol.FileIntf) bool {
-		f := fi.(db.FileInfoTruncated)
-		fs = append(fs, f)
+	snap.WithPrefixedGlobalTruncated(prefix, func(fi protocol.FileInfo) bool {
+		fs = append(fs, fi)
 		return true
 	})
 	return fs
@@ -72,21 +72,19 @@ func haveList(t testing.TB, s *db.FileSet, n protocol.DeviceID) []protocol.FileI
 	var fs []protocol.FileInfo
 	snap := snapshot(t, s)
 	defer snap.Release()
-	snap.WithHave(n, func(fi protocol.FileIntf) bool {
-		f := fi.(protocol.FileInfo)
-		fs = append(fs, f)
+	snap.WithHave(n, func(fi protocol.FileInfo) bool {
+		fs = append(fs, fi)
 		return true
 	})
 	return fs
 }
 
-func haveListPrefixed(t testing.TB, s *db.FileSet, n protocol.DeviceID, prefix string) []db.FileInfoTruncated {
-	var fs []db.FileInfoTruncated
+func haveListPrefixed(t testing.TB, s *db.FileSet, n protocol.DeviceID, prefix string) []protocol.FileInfo {
+	var fs []protocol.FileInfo
 	snap := snapshot(t, s)
 	defer snap.Release()
-	snap.WithPrefixedHaveTruncated(n, prefix, func(fi protocol.FileIntf) bool {
-		f := fi.(db.FileInfoTruncated)
-		fs = append(fs, f)
+	snap.WithPrefixedHaveTruncated(n, prefix, func(fi protocol.FileInfo) bool {
+		fs = append(fs, fi)
 		return true
 	})
 	return fs
@@ -96,9 +94,8 @@ func needList(t testing.TB, s *db.FileSet, n protocol.DeviceID) []protocol.FileI
 	var fs []protocol.FileInfo
 	snap := snapshot(t, s)
 	defer snap.Release()
-	snap.WithNeed(n, func(fi protocol.FileIntf) bool {
-		f := fi.(protocol.FileInfo)
-		fs = append(fs, f)
+	snap.WithNeed(n, func(fi protocol.FileInfo) bool {
+		fs = append(fs, fi)
 		return true
 	})
 	return fs
@@ -106,16 +103,8 @@ func needList(t testing.TB, s *db.FileSet, n protocol.DeviceID) []protocol.FileI
 
 type fileList []protocol.FileInfo
 
-func (l fileList) Len() int {
-	return len(l)
-}
-
-func (l fileList) Less(a, b int) bool {
-	return l[a].Name < l[b].Name
-}
-
-func (l fileList) Swap(a, b int) {
-	l[a], l[b] = l[b], l[a]
+func compareByName(a, b protocol.FileInfo) int {
+	return strings.Compare(a.Name, b.Name)
 }
 
 func (l fileList) String() string {
@@ -222,7 +211,7 @@ func TestGlobalSet(t *testing.T) {
 		t.Helper()
 
 		g := fileList(globalList(t, m))
-		sort.Sort(g)
+		slices.SortFunc(g, compareByName)
 
 		if fmt.Sprint(g) != fmt.Sprint(expectedGlobal) {
 			t.Errorf("Global incorrect;\n A: %v !=\n E: %v", g, expectedGlobal)
@@ -259,7 +248,7 @@ func TestGlobalSet(t *testing.T) {
 		}
 
 		h := fileList(haveList(t, m, protocol.LocalDeviceID))
-		sort.Sort(h)
+		slices.SortFunc(h, compareByName)
 
 		if fmt.Sprint(h) != fmt.Sprint(localTot) {
 			t.Errorf("Have incorrect (local);\n A: %v !=\n E: %v", h, localTot)
@@ -296,14 +285,14 @@ func TestGlobalSet(t *testing.T) {
 		}
 
 		h = fileList(haveList(t, m, remoteDevice0))
-		sort.Sort(h)
+		slices.SortFunc(h, compareByName)
 
 		if fmt.Sprint(h) != fmt.Sprint(remoteTot) {
 			t.Errorf("Have incorrect (remote);\n A: %v !=\n E: %v", h, remoteTot)
 		}
 
 		n := fileList(needList(t, m, protocol.LocalDeviceID))
-		sort.Sort(n)
+		slices.SortFunc(n, compareByName)
 
 		if fmt.Sprint(n) != fmt.Sprint(expectedLocalNeed) {
 			t.Errorf("Need incorrect (local);\n A: %v !=\n E: %v", n, expectedLocalNeed)
@@ -312,7 +301,7 @@ func TestGlobalSet(t *testing.T) {
 		checkNeed(t, m, protocol.LocalDeviceID, expectedLocalNeed)
 
 		n = fileList(needList(t, m, remoteDevice0))
-		sort.Sort(n)
+		slices.SortFunc(n, compareByName)
 
 		if fmt.Sprint(n) != fmt.Sprint(expectedRemoteNeed) {
 			t.Errorf("Need incorrect (remote);\n A: %v !=\n E: %v", n, expectedRemoteNeed)
@@ -432,14 +421,14 @@ func TestGlobalSet(t *testing.T) {
 	check()
 
 	h := fileList(haveList(t, m, remoteDevice1))
-	sort.Sort(h)
+	slices.SortFunc(h, compareByName)
 
 	if fmt.Sprint(h) != fmt.Sprint(secRemote) {
 		t.Errorf("Have incorrect (secRemote);\n A: %v !=\n E: %v", h, secRemote)
 	}
 
 	n := fileList(needList(t, m, remoteDevice1))
-	sort.Sort(n)
+	slices.SortFunc(n, compareByName)
 
 	if fmt.Sprint(n) != fmt.Sprint(expectedSecRemoteNeed) {
 		t.Errorf("Need incorrect (secRemote);\n A: %v !=\n E: %v", n, expectedSecRemoteNeed)
@@ -479,7 +468,7 @@ func TestNeedWithInvalid(t *testing.T) {
 	replace(s, remoteDevice1, remote1Have)
 
 	need := fileList(needList(t, s, protocol.LocalDeviceID))
-	sort.Sort(need)
+	slices.SortFunc(need, compareByName)
 
 	if fmt.Sprint(need) != fmt.Sprint(expectedNeed) {
 		t.Errorf("Need incorrect;\n A: %v !=\n E: %v", need, expectedNeed)
@@ -507,7 +496,7 @@ func TestUpdateToInvalid(t *testing.T) {
 	replace(s, protocol.LocalDeviceID, localHave)
 
 	have := fileList(haveList(t, s, protocol.LocalDeviceID))
-	sort.Sort(have)
+	slices.SortFunc(have, compareByName)
 
 	if fmt.Sprint(have) != fmt.Sprint(localHave) {
 		t.Errorf("Have incorrect before invalidation;\n A: %v !=\n E: %v", have, localHave)
@@ -523,8 +512,8 @@ func TestUpdateToInvalid(t *testing.T) {
 
 	s.Update(protocol.LocalDeviceID, append(fileList{}, localHave[1], localHave[4]))
 
-	have = fileList(haveList(t, s, protocol.LocalDeviceID))
-	sort.Sort(have)
+	have = haveList(t, s, protocol.LocalDeviceID)
+	slices.SortFunc(have, compareByName)
 
 	if fmt.Sprint(have) != fmt.Sprint(localHave) {
 		t.Errorf("Have incorrect after invalidation;\n A: %v !=\n E: %v", have, localHave)
@@ -609,7 +598,7 @@ func TestGlobalReset(t *testing.T) {
 
 	replace(m, protocol.LocalDeviceID, local)
 	g := globalList(t, m)
-	sort.Sort(fileList(g))
+	slices.SortFunc(g, compareByName)
 
 	if diff, equal := messagediff.PrettyDiff(local, g); !equal {
 		t.Errorf("Global incorrect;\nglobal: %v\n!=\nlocal: %v\ndiff:\n%s", g, local, diff)
@@ -619,7 +608,7 @@ func TestGlobalReset(t *testing.T) {
 	replace(m, remoteDevice0, nil)
 
 	g = globalList(t, m)
-	sort.Sort(fileList(g))
+	slices.SortFunc(g, compareByName)
 
 	if diff, equal := messagediff.PrettyDiff(local, g); !equal {
 		t.Errorf("Global incorrect;\nglobal: %v\n!=\nlocal: %v\ndiff:\n%s", g, local, diff)
@@ -657,8 +646,8 @@ func TestNeed(t *testing.T) {
 
 	need := needList(t, m, protocol.LocalDeviceID)
 
-	sort.Sort(fileList(need))
-	sort.Sort(fileList(shouldNeed))
+	slices.SortFunc(need, compareByName)
+	slices.SortFunc(shouldNeed, compareByName)
 
 	if fmt.Sprint(need) != fmt.Sprint(shouldNeed) {
 		t.Errorf("Need incorrect;\n%v !=\n%v", need, shouldNeed)
@@ -1011,9 +1000,9 @@ func TestWithHaveSequence(t *testing.T) {
 	i := 2
 	snap := snapshot(t, s)
 	defer snap.Release()
-	snap.WithHaveSequence(int64(i), func(fi protocol.FileIntf) bool {
-		if f := fi.(protocol.FileInfo); !f.IsEquivalent(localHave[i-1], 0) {
-			t.Fatalf("Got %v\nExpected %v", f, localHave[i-1])
+	snap.WithHaveSequence(int64(i), func(fi protocol.FileInfo) bool {
+		if !fi.IsEquivalent(localHave[i-1], 0) {
+			t.Fatalf("Got %v\nExpected %v", fi, localHave[i-1])
 		}
 		i++
 		return true
@@ -1062,7 +1051,7 @@ loop:
 		default:
 		}
 		snap := snapshot(t, s)
-		snap.WithHaveSequence(prevSeq+1, func(fi protocol.FileIntf) bool {
+		snap.WithHaveSequence(prevSeq+1, func(fi protocol.FileInfo) bool {
 			if fi.SequenceNo() < prevSeq+1 {
 				t.Fatal("Skipped ", prevSeq+1, fi.SequenceNo())
 			}
@@ -1540,8 +1529,8 @@ func TestSequenceIndex(t *testing.T) {
 
 	// Start a routine to walk the sequence index and inspect the result.
 
-	seen := make(map[string]protocol.FileIntf)
-	latest := make([]protocol.FileIntf, 0, len(local))
+	seen := make(map[string]protocol.FileInfo)
+	latest := make([]protocol.FileInfo, 0, len(local))
 	var seq int64
 	t0 := time.Now()
 
@@ -1552,7 +1541,7 @@ func TestSequenceIndex(t *testing.T) {
 		// update has happened since our last iteration.
 		latest = latest[:0]
 		snap := snapshot(t, s)
-		snap.WithHaveSequence(seq+1, func(f protocol.FileIntf) bool {
+		snap.WithHaveSequence(seq+1, func(f protocol.FileInfo) bool {
 			seen[f.FileName()] = f
 			latest = append(latest, f)
 			seq = f.SequenceNo()
@@ -1657,7 +1646,7 @@ func TestUpdateWithOneFileTwice(t *testing.T) {
 	snap := snapshot(t, s)
 	defer snap.Release()
 	count := 0
-	snap.WithHaveSequence(0, func(f protocol.FileIntf) bool {
+	snap.WithHaveSequence(0, func(_ protocol.FileInfo) bool {
 		count++
 		return true
 	})
