@@ -8,6 +8,7 @@ package model
 
 import (
 	"path/filepath"
+	"slices"
 	"testing"
 
 	"github.com/d4l3k/messagediff"
@@ -16,6 +17,7 @@ import (
 	"github.com/syncthing/syncthing/lib/config"
 	"github.com/syncthing/syncthing/lib/fs"
 	"github.com/syncthing/syncthing/lib/protocol"
+	"github.com/syncthing/syncthing/lib/rand"
 )
 
 type unifySubsCase struct {
@@ -116,20 +118,11 @@ func unifySubsCases() []unifySubsCase {
 	return cases
 }
 
-func unifyExists(f string, tc unifySubsCase) bool {
-	for _, e := range tc.exists {
-		if f == e {
-			return true
-		}
-	}
-	return false
-}
-
 func TestUnifySubs(t *testing.T) {
 	cases := unifySubsCases()
 	for i, tc := range cases {
 		exists := func(f string) bool {
-			return unifyExists(f, tc)
+			return slices.Contains(tc.exists, f)
 		}
 		out := unifySubs(tc.in, exists)
 		if diff, equal := messagediff.PrettyDiff(tc.out, out); !equal {
@@ -145,7 +138,7 @@ func BenchmarkUnifySubs(b *testing.B) {
 	for i := 0; i < b.N; i++ {
 		for _, tc := range cases {
 			exists := func(f string) bool {
-				return unifyExists(f, tc)
+				return slices.Contains(tc.exists, f)
 			}
 			unifySubs(tc.in, exists)
 		}
@@ -156,8 +149,7 @@ func TestSetPlatformData(t *testing.T) {
 	// Checks that setPlatformData runs without error when applied to a temp
 	// file, named differently than the given FileInfo.
 
-	dir := t.TempDir()
-	fs := fs.NewFilesystem(fs.FilesystemTypeBasic, dir)
+	fs := fs.NewFilesystem(fs.FilesystemTypeFake, rand.String(32))
 	if fd, err := fs.Create("file.tmp"); err != nil {
 		t.Fatal(err)
 	} else {
@@ -167,7 +159,7 @@ func TestSetPlatformData(t *testing.T) {
 	xattr := []protocol.Xattr{{Name: "user.foo", Value: []byte("bar")}}
 	fi := &protocol.FileInfo{
 		Name:        "should be ignored",
-		Permissions: 0400,
+		Permissions: 0o400,
 		ModifiedS:   1234567890,
 		Platform: protocol.PlatformData{
 			Linux:   &protocol.XattrData{Xattrs: xattr},
